/*
 * Purrpackage - http://purrpackage.sourceforge.net
 * Copyright (C) 2010 John Charles Roth
 *
 * Purrpackage is free software, licensed under the terms of the GNU GPL 
 * Version 2 or, at your option, any later version. You should have 
 * received a copy of the license with this file. See the above web address
 * for more information, or contact the Free Software Foundation, Boston, MA. 
 * It is distributed WITHOUT WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
package net.sourceforge.purrpackage.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import net.sourceforge.cobertura.coveragedata.ClassData;
import net.sourceforge.cobertura.coveragedata.CoverageData;
import net.sourceforge.cobertura.coveragedata.LineData;

public class MethodData implements CoverageData, Comparable<MethodData> {

	ClassData data;
	List<LineData> lines;
	String name;

	public MethodData(ClassData classData,
			String methodNameAndDescriptor) {
		this.lines = new ArrayList<LineData>( );
		lines.addAll( (Collection) classData.getLines(methodNameAndDescriptor) );
		Collections.sort( this.lines, new Comparator<CoverageData>() {

			@Override
			public int compare(CoverageData o1, CoverageData o2) {
				LineData l1 = (LineData) o1;
				LineData l2 = (LineData) o2;
				Integer i1 = new Integer( l1.getLineNumber() );
				Integer i2 = new Integer( l2.getLineNumber() );
				return i1.compareTo( i2 );
			}
			
		});
		this.data = classData;
		name = methodNameAndDescriptor;
	}

	public Collection<LineData> getLines() {
		return lines;
	}
	
	public String getName() {
		return name;
	}

	@Override
	public double getBranchCoverageRate() {
		return (double) getNumberOfCoveredBranches() / (double) getNumberOfValidBranches();
	}

	@Override
	public double getLineCoverageRate() {
		return (double) getNumberOfCoveredLines() / (double) getNumberOfValidLines();
	}

	@Override
	public int getNumberOfCoveredBranches() {
		int result = 0;
		for ( CoverageData cd : lines ) {
			result += cd.getNumberOfCoveredBranches();
		}
		return result;
	}

	@Override
	public int getNumberOfCoveredLines() {
		int result = 0;
		for ( CoverageData cd : lines ) {
			result += cd.getNumberOfCoveredLines();
		}
		return result;
	}

	@Override
	public int getNumberOfValidBranches() {
		int result = 0;
		for ( CoverageData cd : lines ) {
			result += cd.getNumberOfValidBranches();
		}
		return result;
	}

	@Override
	public int getNumberOfValidLines() {
		int result = 0;
		for ( CoverageData cd : lines ) {
			result += cd.getNumberOfValidLines();
		}
		return result;

	}

	@Override
	public void merge(CoverageData arg0) {
		throw new UnsupportedOperationException();		
	}
	
	public int[] getFirstAndLastLinesForMethod() {
		int[] result = new int[] { Integer.MAX_VALUE, Integer.MIN_VALUE };
		for( CoverageData cd : lines ) {
			LineData ld = (LineData) cd;
			result[0] = Math.min( result[0], ld.getLineNumber() );
			result[1] = Math.max( result[1], ld.getLineNumber() );
		}
		return result;
	}

	@Override
	public int compareTo(MethodData x) {
		int a = getFirstAndLastLinesForMethod()[0];
		int b = x.getFirstAndLastLinesForMethod()[0];
		return a - b;
	}

}
