/*
 * Purrpackage - http://purrpackage.sourceforge.net
 * Copyright (C) 2010 John Charles Roth
 *
 * Purrpackage is free software, licensed under the terms of the GNU GPL 
 * Version 2 or, at your option, any later version. You should have 
 * received a copy of the license with this file. See the above web address
 * for more information, or contact the Free Software Foundation, Boston, MA. 
 * It is distributed WITHOUT WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
package net.sourceforge.purrpackage.parsing;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;

import net.sourceforge.cobertura.coveragedata.ProjectData;

/**
 * Computes the differences between the elements on an increasing sequence of
 * coverage snapshots for given packages. Merges results for packages that occur
 * more than once.
 */

public class PackageCoverageCalculator {

	List<PackageAndProjectData> smallToLarge;
	public List<PackageAndProjectData> getSmallToLarge() {
		return smallToLarge;
	}

	public SortedMap<String, ProjectData> getPackageToCoverage() {
		return packageToCoverage;
	}

	List<ProjectDataDifferenceCalculator> diffs =  new ArrayList<ProjectDataDifferenceCalculator>();
	public List<ProjectDataDifferenceCalculator> getDiffs() {
		return diffs;
	}

	SortedMap<String,ProjectData> packageToCoverage = new TreeMap<String, ProjectData>();
	ProjectData globalData;
	
	public PackageCoverageCalculator( Collection<PackageAndProjectData> smallToLarge ) {
		this.smallToLarge = new ArrayList<PackageAndProjectData>(smallToLarge);
		compute();
	}
	
	void compute() {
		Iterator<PackageAndProjectData> iterator = smallToLarge.iterator();
		if ( ! iterator.hasNext() ) {
			return;			
		}
		PackageAndProjectData prev = iterator.next(); 
		for (; iterator.hasNext();) {
			PackageAndProjectData ppd = iterator.next();
			ProjectDataDifferenceCalculator calc = new ProjectDataDifferenceCalculator(
					ppd.getData(), prev.getData());
			ProjectData perPackage = packageToCoverage
					.get(ppd.getPackageName());
			if (perPackage == null) {
				perPackage = new ProjectData();
				packageToCoverage.put(ppd.getPackageName(), perPackage);
			} 
			perPackage.merge(calc.getDifference());
			diffs.add(calc);
			prev = ppd;
		}
		globalData = prev.getData();
	}

	public boolean isErrorFree() {
		for( ProjectDataDifferenceCalculator c : getDiffs() ) {
			if ( c.getErrors().size() > 0 ) {
				return false;
			}
		}
		return true;
	}
	
}