/*
 * Purrpackage - http://purrpackage.sourceforge.net
 * Copyright (C) 2010 John Charles Roth
 *
 * Purrpackage is free software, licensed under the terms of the GNU GPL 
 * Version 2 or, at your option, any later version. You should have 
 * received a copy of the license with this file. See the above web address
 * for more information, or contact the Free Software Foundation, Boston, MA. 
 * It is distributed WITHOUT WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
package net.sourceforge.purrpackage.recording;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;


/**
 * Java NIO copy using a {@link Closer}.
 */

public class Copier {

	public void copyFile(File sourceFile, File destFile) throws IOException {
		 if(!destFile.exists()) {
			 destFile.createNewFile();
		 }
		 
		 FileChannel source = null;
		 FileChannel destination = null;
		 Closer c = new Closer();
		 try {
			 source = c.register( new FileInputStream(sourceFile).getChannel() );
			 destination = c.register( new FileOutputStream(destFile).getChannel() );
			 destination.transferFrom(source, 0, source.size());
		 }
		 catch( IOException e ) {
			 c.doNotThrow();
			 throw e;
		 }
		 finally {
			 c.closeAll();
		}
	}
	
}