/*
 * Purrpackage - http://purrpackage.sourceforge.net
 * Copyright (C) 2010 John Charles Roth
 *
 * Purrpackage is free software, licensed under the terms of the GNU GPL 
 * Version 2 or, at your option, any later version. You should have 
 * received a copy of the license with this file. See the above web address
 * for more information, or contact the Free Software Foundation, Boston, MA. 
 * It is distributed WITHOUT WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

package net.sourceforge.purrpackage.reporting;

import java.io.File;
import java.io.PrintStream;

import net.sourceforge.cobertura.coveragedata.CoverageDataFileHandler;
import net.sourceforge.cobertura.util.CommandLineBuilder;

/**
 * For use by ReportTask
 */

public class Main {

	ReportSupport support = new ReportSupport( );
	String encoding = "UTF-8";

	void parseArguments(String[] args) throws Exception {
		for (int i = 0; i < args.length; i++) {
			if (args[i].equals("--datafile")) {
				setDataFile( args[++i] );
			} else if (args[i].equals("--destination")) {
				setDestination(args[++i]);	
			} else if (args[i].equals("--encoding")) {
				support.setEncoding(args[++i]);
			} else if (args[i].equals("--coveragePolicy")) {
				support.setCoveragePolicyPath(args[++i]);
			} else if (args[i].equals("--buildTimeReportScript")) {
				support.setBuildTimeReportScriptPath(args[++i]);
			} else {
				support.addSourceDirectory( args[i] );
			}
		}
		if (support.getCoverageDataFile() == null)
			support.setCoverageDataFile(CoverageDataFileHandler.getDefaultDataFile());

		if ( support.getDestinationDir() == null) {
			throw new RuntimeException( "Error: destination directory must be set");
		}
	}

	void readAndParseCoverageData() throws Exception {
		support.readAndParseCoverageData();
		JsonReport jr = createJsonReport();
		jr.report();
	}

	JsonReport createJsonReport() {
		return new JsonReport( support );
	}
	
	private void setDataFile(String value) {
		File dataFile = new File( value );
		support.setCoverageDataFile( dataFile );
	}

	private void setDestination(String value) {
		File destinationDir = new File(value);
		support.setDestinationDir( destinationDir );
	}

	public static void printHeader(PrintStream out) {
		out.println( "Purrpackage extension to Cobertura - (c) 2010 - GPL - No Warranty" );		
	}
	
	public static void main(String[] args) throws Exception {
		printHeader(System.out);
		long startTime = System.currentTimeMillis();
		Main main = new Main();
		args = CommandLineBuilder.preprocessCommandLineArguments( args);
		main.parseArguments(args);
		main.readAndParseCoverageData();
		long stopTime = System.currentTimeMillis();
		System.out.println("Report time: " + (stopTime - startTime) + "ms");
	}

}
