/*
 * Purrpackage - http://purrpackage.sourceforge.net
 * Copyright (C) 2010 John Charles Roth
 *
 * Purrpackage is free software, licensed under the terms of the GNU GPL 
 * Version 2 or, at your option, any later version. You should have 
 * received a copy of the license with this file. See the above web address
 * for more information, or contact the Free Software Foundation, Boston, MA. 
 * It is distributed WITHOUT WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

package net.sourceforge.purrpackage.reporting;

import java.io.File;
import java.io.IOException;

import net.sourceforge.cobertura.ant.CommonMatchingTask;
import net.sourceforge.cobertura.util.CommandLineBuilder;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Java;
import org.apache.tools.ant.types.Environment.Variable;

/**
 * Ant task to generate a coverage report based on coverage data generated by
 * instrumented classes.
 */
public class ReportTask extends CommonMatchingTask {

	private String dataFile;
	private String format = "json";
	private File destDir;
	private String srcDir;
    private String encoding;
    private String coveragePolicy;
	private String buildTimeReportScript;
    
	public ReportTask() {
		super( Main.class.getName() );
	}

	CommandLineBuilder makeCommandLineBuilder() throws IOException {
		return new CommandLineBuilder();
	}
	
	public void execute() throws BuildException {
		CommandLineBuilder builder = null;
		try {
			builder = makeCommandLineBuilder();
			if (dataFile != null)
				builder.addArg("--datafile", dataFile);
			if (destDir != null)
				builder.addArg("--destination", destDir.getAbsolutePath());
			if (format != null)
				builder.addArg("--format", format);
			if (coveragePolicy != null) {
				builder.addArg("--coveragePolicy", coveragePolicy);
			}
			if (buildTimeReportScript != null ) {
				builder.addArg("--buildTimeReportScript", buildTimeReportScript );
			}
			if (encoding != null)
				builder.addArg("--encoding", encoding);
			if (srcDir != null) {
				String[] dirNames = srcDir.split(",");
				for (String x : dirNames) {
					builder.addArg(x);
				}
			}
			createArgumentsForFilesets(builder);

			builder.saveArgs();
		} catch (IOException ioe) {
			getProject().log("Error creating commands file.", Project.MSG_ERR);
			throw new BuildException("Unable to create the commands file.", ioe);
		}

		// Execute GPL licensed code in separate virtual machine
		getJava().createArg().setValue("--commandsfile");
		getJava().createArg().setValue(builder.getCommandLineFile());
		getJava().setOutput(new File("purrPackageReporter.out"));
		getJava().setError(new File("purrPackageReporter.err"));
		transferCoberturaDataFileProperty(getJava());
		if (getJava().executeJava() != 0) {
			throw new BuildException(
					"Error running reports. See messages above.");
		}

		builder.dispose();
	}

	String COB_PROP_KEY = "net.sourceforge.cobertura.datafile";

	String coberturaPropertyValue() {
		return System.getProperty( COB_PROP_KEY );
	}
	
	void transferCoberturaDataFileProperty(Java task) {
		String coberturaProperty = coberturaPropertyValue();
		if (coberturaProperty != null) {
			Variable sysproperty = new Variable();
			sysproperty.setKey( COB_PROP_KEY);
			sysproperty.setValue(coberturaProperty);
			task.addSysproperty(sysproperty);
		}
	}

	public void setCoveragePolicy(String path) {
		this.coveragePolicy = path;
	}

	public void setDataFile(String dataFile) {
		this.dataFile = dataFile;
	}

	public void setDestDir(File destDir) {
		this.destDir = destDir;
	}

	public void setFormat(String format) {
		this.format = format;
	}

	public void setEncoding(String encoding) {
		this.encoding = encoding;
	}
	
	public void setSrcDir(String dir) {
		srcDir = dir;
	}
	
	public void setBuildTimeReportScript( String path ) {
		buildTimeReportScript = path;
	}
}
