/*
 * Purrpackage - http://purrpackage.sourceforge.net
 * Copyright (C) 2010 John Charles Roth
 *
 * Purrpackage is free software, licensed under the terms of the GNU GPL 
 * Version 2 or, at your option, any later version. You should have 
 * received a copy of the license with this file. See the above web address
 * for more information, or contact the Free Software Foundation, Boston, MA. 
 * It is distributed WITHOUT WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
package net.sourceforge.purrpackage.reporting;
import java.io.BufferedReader;
import java.io.Closeable;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;

import net.sourceforge.cobertura.util.FileFinder;
import net.sourceforge.cobertura.util.Source;
import net.sourceforge.purrpackage.recording.Closer;


public class SourceReader {
	
	public static interface Callback {
		public void lineRead(String line) throws Exception;
	}

	static class SourceReaderException extends RuntimeException {
		private static final long serialVersionUID = 1L;

		public SourceReaderException( String message ) {
			super( message );
		}
	}
	
	public SourceReader( FileFinder finder, String encoding ) {
		this.finder = finder;
		this.encoding = encoding;
	}
	String encoding;
	FileFinder finder;
	public void read( String sourceFileName, Callback callback ) throws Exception {

		Closer c = new Closer();
		final Source source = finder
				.getSource(sourceFileName);
		if (source == null) {
			throw new SourceReaderException( "Could not find source file for "
					+ sourceFileName);
		}
		c.register( new Closeable() {
			
			@Override
			public void close() throws IOException {
				source.close();				
			}
		} );

		BufferedReader br = null;
		try {
			br = readerFor( source );
			c.register( br );
		} catch (UnsupportedEncodingException e) {
			throw new SourceReaderException( "Encoding problem for source file "
					+ sourceFileName + ": "
					+ e.getMessage());
		} catch (Throwable t) {
			throw new SourceReaderException( "Could not create reader for source file "
							+ sourceFileName + ": "
							+ t.toString());
		}

		try {
			String lineStr;
			while ((lineStr = br.readLine()) != null) {
				callback.lineRead( lineStr );
			}
		} catch (Exception e) {
			c.doNotThrow();
			throw new SourceReaderException( "Could not read "
							+ sourceFileName + ": "
							+ e.toString());
		} finally {
			c.closeAll();
		}
	}

	BufferedReader readerFor( Source source ) throws UnsupportedEncodingException {
		return new BufferedReader( new InputStreamReader( source.getInputStream(), encoding));
	}
	
}
