/*
 * Purrpackage - http://purrpackage.sourceforge.net
 * Copyright (C) 2010 John Charles Roth
 *
 * Purrpackage is free software, licensed under the terms of the GNU GPL 
 * Version 2 or, at your option, any later version. You should have 
 * received a copy of the license with this file. See the above web address
 * for more information, or contact the Free Software Foundation, Boston, MA. 
 * It is distributed WITHOUT WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

package net.sourceforge.purrpackage.reporting.json;

import java.util.HashMap;
import java.util.Map;

import net.sourceforge.purrpackage.reporting.json.org.json.JSONObject;

public class JsonFormatter {
	
	Map<Class<?>, JsonFormatterElement<?>> formatters = new HashMap<Class<?>, JsonFormatterElement<?>>();
	
	public <T> void register( Class<T> klass, JsonFormatterElement<T> formatter ) {
		formatters.put( klass, formatter );
	}

	public JSONObject format(Object object) throws Exception {
		JSONObject result = null;
		if (object != null ) {
			JsonFormatterElement x = findFor( object.getClass() );
			if (x != null ) {
				result = x.format(object, this);
			}
		}
		return result;
	}
	
	JsonFormatterElement<?> findFor( Class<?> t ) {
		JsonFormatterElement<?> result = formatters.get( t );
		if ( result == null ) {
			if ( t.getSuperclass() != null ) {
				result = findFor( t.getSuperclass() );
			}
		}
		return result;
	}
}
