/*
 * Purrpackage - http://purrpackage.sourceforge.net
 * Copyright (C) 2010 John Charles Roth
 *
 * Purrpackage is free software, licensed under the terms of the GNU GPL 
 * Version 2 or, at your option, any later version. You should have 
 * received a copy of the license with this file. See the above web address
 * for more information, or contact the Free Software Foundation, Boston, MA. 
 * It is distributed WITHOUT WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
package net.sourceforge.purrpackage.reporting.json;


import java.util.concurrent.atomic.AtomicInteger;

import net.sourceforge.cobertura.coveragedata.PackageData;
import net.sourceforge.cobertura.coveragedata.ProjectData;
import net.sourceforge.cobertura.coveragedata.SourceFileData;
import net.sourceforge.cobertura.reporting.ComplexityCalculator;
import net.sourceforge.purrpackage.model.CoverageDataPair;
import net.sourceforge.purrpackage.model.CoverageSummaryView;
import net.sourceforge.purrpackage.model.MethodDataPair;
import net.sourceforge.purrpackage.model.PackageDataPair;
import net.sourceforge.purrpackage.model.ProjectDataPair;
import net.sourceforge.purrpackage.model.SourceFileDataPair;
import net.sourceforge.purrpackage.parsing.PerPackageCoverageRunSummary;
import net.sourceforge.purrpackage.reporting.SourceReader;
import net.sourceforge.purrpackage.reporting.SourceReader.Callback;
import net.sourceforge.purrpackage.reporting.json.org.json.JSONObject;


public class MainJsonFormatter extends JsonFormatter {

	public MainJsonFormatter( final ComplexityCalculator complexityCalc, final SourceReader reader ) {
		register(CoverageSummaryView.class, new CoverageSummaryViewJsonFormatterElement() );
		register(PerPackageCoverageRunSummary.class,
				new JsonFormatterElement<PerPackageCoverageRunSummary>() {

					@Override
					public JSONObject format(PerPackageCoverageRunSummary x,
							JsonFormatter utility) throws Exception {
						JSONObject jo = new JSONObject();
						jo.put("filesRead", x.getFilesRead());
						jo.put("errorFree", x.isErrorFree());
						jo.put("readerErrors", x.getReaderErrors());
						jo.put("calculatorErrors", x.getCalculatorErrors());
						jo.put("data", utility.format(x.getData()));
						return jo;
					}
				});
		
		register(CoverageDataPair.class,
				new JsonFormatterElement<CoverageDataPair>() {

					@Override
					public JSONObject format(CoverageDataPair x,
							JsonFormatter utility) throws Exception {
						JSONObject result = new JSONObject();
						result.put("elementCounts", utility.format(x
								.getElementCounts()));
						result.put("branchCounts", utility.format(x
								.getBranchCounts()));
						result.put("lineCounts", utility.format(x
								.getLineCounts()));
						result.put("displayName", x.getDisplayName());
						result.put("orderingObject", x.getOrderingObject());
						result.put("name", x.getName());
						result.put("childKeys", x.getChildKeys());
						final AtomicInteger sourceLineCount = new AtomicInteger( 0 );
						if ( x instanceof SourceFileDataPair ) {
							try {
							reader.read( ((SourceFileDataPair) x).getData().getName(), new Callback() {
								
								@Override // just a line counter
								public void lineRead(String line) throws Exception {
									sourceLineCount.incrementAndGet();
								}								
							});
							}
							catch( Exception e ) {
								// detailed problems reading the source file are recorded elsewhere.
								sourceLineCount.set( 0 ); 
							}
						}
						result.put( "sourceFileLineCount", sourceLineCount.get() );
						if (!(x instanceof MethodDataPair)) {
							JSONObject children = new JSONObject();
							for (Object o : x.getChildKeys()) {
								children.put(String.valueOf(o), utility
										.format(x.getChild(o)));
							}
							result.put("children", children);
						}
						Double complexity = null;
						if ( x instanceof SourceFileDataPair ) {
							SourceFileData sfd = (SourceFileData) x.getData();
							complexity = complexityCalc.getCCNForSourceFile( sfd );
						}
						if ( x instanceof PackageDataPair ) {
							PackageData pd = (PackageData) x.getData();
							complexity = complexityCalc.getCCNForPackage( pd );
						}
						if ( x instanceof ProjectDataPair ) {
							ProjectData pjd = (ProjectData) x.getData();
							complexity = complexityCalc.getCCNForProject( pjd );
						}
						if (complexity != null ) {
							result.put( "complexity", complexity.doubleValue() );
						}
						else {
							result.put( "complexity", (Object) null );
						}
						return result;
					}

				});
	}

}
